/*
 * Copyright (c) 2007 Alexander Wiedenbruch <mail@wiedenbruch.de>
 *
 * This file is part of SuperKaramba.
 *
 *  SuperKaramba is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  SuperKaramba is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SuperKaramba; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 ****************************************************************************/

#ifndef KARAMBAINTERFACE_H_
#define KARAMBAINTERFACE_H_

#include <QObject>

#include <kross/core/manager.h>
#include <kross/core/guiclient.h>
#include <kross/core/action.h>

#include "karamba.h"
#include "meters/bar.h"
#include "meters/graph.h"
#include "meters/imagelabel.h"
#include "meters/input.h"
#include "meters/richtextlabel.h"
#include "meters/textlabel.h"

class Karamba;
class ImageLabel;
class Input;
class RichTextLabel;
class TextLabel;

class KarambaInterface : public QObject
{
  Q_OBJECT

  public:
    KarambaInterface(Karamba *k);
    virtual ~KarambaInterface();

    // Calls to scripts ----------------------
    void callInitWidget(Karamba *k);
    void callWidgetUpdated(Karamba *k);
    void callWidgetClosed(Karamba *k);
    void callMenuOptionChanged(Karamba *k, QString key, bool value);
    void callMenuItemClicked(Karamba* k, KMenu* menu, int id);
    void callActiveTaskChanged(Karamba *k, Task* t);
    void callTaskAdded(Karamba *k, Task *t);
    void callTaskRemoved(Karamba *k, Task *t);
    void callStartupAdded(Karamba *k, Startup *t);
    void callStartupRemoved(Karamba *k, Startup *t);
    void callCommandFinished(Karamba *k, int pid);
    void callCommandOutput(Karamba *k, int pid, char* buffer);
    void callItemDropped(Karamba *k, QString text, int x, int y);
    void callMeterClicked(Karamba *k, Meter *m, int button);
    void callMeterClicked(Karamba *k, QString str, int button);
    void callWidgetClicked(Karamba *k, int x, int y, int button);
    void callDesktopChanged(Karamba *k, int desktop);
    void callWidgetMouseMoved(Karamba *k, int x, int y, int button);
    void callKeyPressed(Karamba *k, Meter *meter, QString key);

  Q_SIGNALS:
    void initWidget(QObject*);
    void widgetUpdated(QObject*);
    void widgetClosed(QObject*);
    void menuItemClicked(QObject*, QObject*, int id);
    void menuOptionChanged(QObject*, QString, bool);
    void activeTaskChanged(QObject*, long long);
    void taskAdded(QObject*, long long);
    void taskRemoved(QObject*, long long);
    void startupAdded(QObject*, long long);
    void startupRemoved(QObject*, long long);
    void commandFinished(QObject*, int);
    void commandOutput(QObject*, int, char*);
    void itemDropped(QObject*, QString, int, int);
    void meterClicked(QObject*, QObject*, int);
    void meterClicked(QObject*, QString, int);
    void widgetClicked(QObject*, int, int, int);
    void desktopChanged(QObject*, int);
    void widgetMouseMoved(QObject*, int, int, int);
    void keyPressed(QObject*, QObject*, QString);

  private:
    Karamba *m_karamba;

    Kross::Action *m_action;

    bool checkKaramba(Karamba *k);
    bool checkMeter(Karamba *k, Meter *m, QString type);
    bool checkKarambaAndMeter(Karamba *k, Meter *m, QString type);
    
    QVariantList getMeterMinMax(Karamba *k, Meter *m, QString type);
    QVariantList getMeterSize(Karamba *k, Meter *m, QString type);
    QVariantList getMeterPos(Karamba *k, Meter *m, QString type);
    QString getMeterSensor(Karamba *k, Meter *m, QString type);
    int getMeterValue(Karamba *k, Meter *m, QString type);
    QObject* getThemeMeter(Karamba *k, QString meter, QString type);
    bool hideMeter(Karamba *k, Meter *m, QString type);
    bool moveMeter(Karamba *k, Meter *m, QString type, int x, int y);
    bool resizeMeter(Karamba *k, Meter *m, QString type, int width, int height);
    bool setMeterMinMax(Karamba *k, Meter *m, QString type, int min, int max);
    bool setMeterSensor(Karamba *k, Meter *m, QString type, QString sensor);
    QObject* setMeterValue(Karamba *k, Meter *m, QString type, int value);
    bool showMeter(Karamba *k, Meter *m, QString type);
    bool setMeterColor(Karamba *k, Meter *m, QString type, int red, int green, int blue);
    QVariantList getMeterColor(Karamba *k, Meter *m, QString type);
    QString getMeterStringValue(Karamba *k, Meter *m, QString type);
    QObject* setMeterStringValue(Karamba *k, Meter *m, QString type, QString value);
    bool menuExists(Karamba* currTheme, KMenu* menu);



  public Q_SLOTS:
    // Bar
    QObject* createBar(Karamba *k, int x, int y, int w, int h, QString path = "");
    bool deleteBar(Karamba *k, Bar *bar);
    bool setBarMinMax(Karamba *k, Bar *bar, int min, int max);  
    QVariantList getBarMinMax(Karamba *k, Bar *bar);
    bool moveBar(Karamba *k, Bar *bar, int x, int y);
    QVariantList getBarPos(Karamba *k, Bar *bar);
    bool setBarSensor(Karamba *k, Bar *bar, QString sensor);
    QString getBarSensor(Karamba *k, Bar *bar);
    bool resizeBar(Karamba *k, Bar *bar, int width, int height);
    QVariantList getBarSize(Karamba *k, Bar *bar);
    QObject* setBarValue(Karamba *k, Bar *bar, int value);
    int getBarValue(Karamba *k, Bar *bar);
    QObject* getThemeBar(Karamba *k, QString meter);
    bool hideBar(Karamba *k, Bar *bar);
    bool showBar(Karamba *k, Bar *bar);
    //---------------------------------------------------------------
    bool setBarVertical(Karamba *k, Bar *bar, bool vert);
    bool getBarVertical(Karamba *k, Bar *bar);
    bool setBarImage(Karamba *k, Bar *bar, QString image);
    QString getBarImage(Karamba *k, Bar *bar);
    
    
    // Config
    bool addMenuConfigOption(Karamba *k, QString key, QString name);
    QString readConfigEntry(Karamba *k, QString key);
    bool readMenuConfigOption(Karamba *k, QString key);
    bool setMenuConfigOption(Karamba *k, QString key, bool value);
    bool writeConfigEntry(Karamba *k, QString key, QString value);

    // Graph
    QObject* createGraph(Karamba* k, int x, int y, int w, int h, int points);
    bool deleteGraph(Karamba *k, Graph *graph);
    bool setGraphMinMax(Karamba *k, Graph *bar, int min, int max);
    QVariantList getGraphMinMax(Karamba *k, Graph *graph);
    bool moveGraph(Karamba *k, Graph *graph, int x, int y);
    QVariantList getGraphPos(Karamba *k, Graph *graph);
    bool setGraphSensor(Karamba *k, Graph *graph, QString sensor);
    QString getGraphSensor(Karamba *k, Graph *graph);
    bool resizeGraph(Karamba *k, Graph *graph, int width, int height);
    QVariantList getGraphSize(Karamba *k, Graph *graph);
    QObject* setGraphValue(Karamba *k, Graph *graph, int value);
    int getGraphValue(Karamba *k, Graph *graph);
    QObject* getThemeGraph(Karamba *k, QString meter);
    bool hideGraph(Karamba *k, Graph *graph);
    bool showGraph(Karamba *k, Graph *graph);
    bool setGraphColor(Karamba *k, Graph *graph, int red, int green, int blue);
    QVariantList getGraphColor(Karamba *k, Graph *graph);

    // ImageLabel
    QObject* createImage(Karamba* k, int x, int y, QString image);
    bool deleteImage(Karamba *k, ImageLabel *image);
    bool setImageMinMax(Karamba *k, ImageLabel *image, int min, int max);
    QVariantList getImageMinMax(Karamba *k, ImageLabel *image);
    bool moveImage(Karamba *k, ImageLabel *image, int x, int y);
    QVariantList getImagePos(Karamba *k, ImageLabel *image);
    bool setImageSensor(Karamba *k, ImageLabel *image, QString sensor);
    QString getImageSensor(Karamba *k, ImageLabel *image);
    bool resizeImage(Karamba *k, ImageLabel *image, int width, int height);
    QVariantList getImageSize(Karamba *k, ImageLabel *image);
    QObject* setImagePath(Karamba *k, ImageLabel *image, QString path);
    QString getImagePath(Karamba *k, ImageLabel *image);
    QObject* getThemeImage(Karamba *k, QString meter);
    bool hideImage(Karamba *k, ImageLabel *image);
    bool showImage(Karamba *k, ImageLabel *image);
    bool setImageColor(Karamba *k, ImageLabel *image, int red, int green, int blue);
    QVariantList getImageColor(Karamba *k, ImageLabel *image);
    bool addImageTooltip(Karamba *k, ImageLabel *image, QString text);
    bool changeImageChannelIntensity(Karamba *k, ImageLabel *image, double ratio, QString channel, int ms = 0);
    bool changeImageIntensity(Karamba *k, ImageLabel *image, double ratio, int ms = 0);
    bool changeImageToGray(Karamba *k, ImageLabel *image, int ms);
    QObject* createBackgroundImage(Karamba *k, int x, int y, QString imagePath);
    QObject* createTaskIcon(Karamba *k, int x, int y, int ctask);
    int getImageHeight(Karamba *k, ImageLabel *image);
    int getImageWidth(Karamba *k, ImageLabel *image);
    bool removeImageEffects(Karamba *k, ImageLabel *image);
    bool removeImageTransformations(Karamba *k, ImageLabel *image);
    bool resizeImageSmooth(Karamba *k, ImageLabel *image, int width, int height);
    bool rotateImage(Karamba *k, ImageLabel *image, int deg);

    // InputBox
    QObject* createInputBox(Karamba* k, int x, int y, int w, int h, QString text);
    bool deleteInputBox(Karamba *k, Input *input);
    bool moveInputBox(Karamba *k, Input *input, int x, int y);
    QVariantList getInputBoxPos(Karamba *k, Input *input);
    bool resizeInputBox(Karamba *k, Input *input, int width, int height);
    QVariantList getInputBoxSize(Karamba *k, Input *input);
    QObject* changeInputBoxValue(Karamba *k, Input *input, QString text);
    QString getInputBoxValue(Karamba *k, Input *input);
    QObject* getThemeInputBox(Karamba *k, QString meter);
    bool hideInputBox(Karamba *k, Input *input);
    bool showInputBox(Karamba *k, Input *input);
    bool changeInputBoxFont(Karamba *k, Input *input, QString font);
    bool changeInputBoxFontColor(Karamba *k, Input *input, int red, int green, int blue);
    QString getInputBoxFont(Karamba *k, Input *input);
    QVariantList getInputBoxFontColor(Karamba *k, Input *input);
    bool changeInputBoxSelectionColor(Karamba *k, Input *input, int red, int green, int blue);
    QVariantList getInputBoxSelectionColor(Karamba *k, Input *input);
    bool changeInputBoxBackgroundColor(Karamba *k, Input *input, int red, int green, int blue);
    QVariantList getInputBoxBackgroundColor(Karamba *k, Input *input);
    bool changeInputBoxFrameColor(Karamba *k, Input *input, int red, int green, int blue);
    QVariantList getInputBoxFrameColor(Karamba *k, Input *input);
    bool changeInputBoxSelectedTextColor(Karamba *k, Input *input, int red, int green, int blue);
    QVariantList getInputBoxSelectedTextColor(Karamba *k, Input *input);
    bool changeInputBoxFontSize(Karamba *k, Input *input, int size);
    int getInputBoxFontSize(Karamba *k, Input *input);
    bool setInputFocus(Karamba *k, Input *input);
    bool clearInputFocus(Karamba *k, Input *input);
    QObject* getInputFocus(Karamba *k);

    // Menu
    QObject* addMenuItem(Karamba *k, KMenu *menu, QString text, QString icon);
    QObject* addMenuSeperator(Karamba *k, KMenu *menu);
    QObject* createMenu(Karamba *k);
    bool deleteMenu(Karamba *k, KMenu *menu);
    bool popupMenu(Karamba *k, KMenu *menu, int x, int y);
    bool removeMenuItem(Karamba *k, KMenu *menu, QAction *action);

/*
    // Misc
    long acceptDrops
    long attachClickArea
    long callTheme
    long changeInterval
    long createClickArea
    long createServiceClickArea
    long execute
    long executeInteractive
    long getIncomingData
    long getIp
    long getNumberOfDesktop
    long getPrettyThemeName
    long getServiceGroups
    long getThemePath
    long getUpdateTime
    long hide
    long language
    long managementPopup
    long openNamedTheme
    long openTheme
    long readThemeFile
    long reloadTheme
    long removeClickArea
    long run
    long setIncomingData
    long setUpdateTime
    long show
    long toggleShowDesktop
    long translateAll
    long userLanguage
    long wantRightButton
*/
    // RichText
    QObject* createRichText(Karamba* k, QString text, bool underline = false);
    bool deleteRichText(Karamba *k, RichTextLabel *label);
    bool moveRichText(Karamba *k, RichTextLabel *label, int x, int y);
    QVariantList getRichTextPos(Karamba *k, RichTextLabel *label);
    bool resizeRichText(Karamba *k, RichTextLabel *label, int width, int height);
    bool setRichTextWidth(Karamba *k, RichTextLabel *label, int width);
    QVariantList getRichTextSize(Karamba *k, RichTextLabel *label);
    QObject* changeRichText(Karamba *k, RichTextLabel *label, QString text);
    QString getRichTextValue(Karamba *k, RichTextLabel *label);
    QObject* getThemeRichText(Karamba *k, QString meter);
    bool hideRichText(Karamba *k, RichTextLabel *label);
    bool showRichText(Karamba *k, RichTextLabel *label);
    bool changeRichTextFont(Karamba *k, RichTextLabel *label, QString font);
    QString getRichTextFont(Karamba *k, RichTextLabel *label);
    bool changeRichTextSize(Karamba *k, RichTextLabel *label, int size);
    int getRichTextFontSize(Karamba *k, RichTextLabel *label);
    bool setRichTextSensor(Karamba *k, RichTextLabel *label, QString sensor);
    QString getRichTextSensor(Karamba *k, RichTextLabel *label);
    
/*
    // Systray
    long createSystray
    long getCurrentWindowCount
    long hideSystray
    long moveSystray
    long showSystray
    long updateSystrayLayout

    // Task
    long getStartupInfo
    long getStartupList
    long getTaskInfo
    long getTaskList
    long getTaskNames
    long performTaskAction
*/
    // Text
    QObject* createText(Karamba* k, int x, int y, int width, int height, QString text);
    bool deleteText(Karamba *k, TextLabel *text);
    bool moveText(Karamba *k, TextLabel *text, int x, int y);
    QVariantList getTextPos(Karamba *k, TextLabel *text);
    bool setTextSensor(Karamba *k, TextLabel *text, QString sensor);
    QString getTextSensor(Karamba *k, TextLabel *text);
    bool resizeText(Karamba *k, TextLabel *text, int width, int height);
    QVariantList getTextSize(Karamba *k, TextLabel *text);
    QObject* getThemeText(Karamba *k, QString meter);
    bool hideText(Karamba *k, TextLabel *text);
    bool showText(Karamba *k, TextLabel *text);
    QObject* changeText(Karamba *k, TextLabel *label, QString text);
    QString getTextValue(Karamba *k, TextLabel *label);
    bool changeTextShadow(Karamba *k, TextLabel *label, int shadow);
    int getTextShadow(Karamba *k, TextLabel *text);
    bool changeTextFont(Karamba *k, TextLabel *text, QString font);
    QString getTextFont(Karamba *k, TextLabel *text);
    bool changeTextColor(Karamba *k, TextLabel *text, int red, int green, int blue);
    QVariantList getTextColor(Karamba *k, TextLabel *text);
    bool changeTextSize(Karamba *k, TextLabel *text, int size);
    int getTextFontSize(Karamba *k, TextLabel *text);
    QString getTextAlign(Karamba *k, TextLabel *text);
    bool setTextAlign(Karamba *k, TextLabel *text, QString alignment);
    bool setTextScroll(Karamba *k, TextLabel *text, QString type, int x = 0, int y = 0, int gap = 0, int pause = 0);
    
    // Widget
    bool createWidgetMask(Karamba *k, QString mask);
    QVariantList getWidgetPostion(Karamba *k);
    bool moveWidget(Karamba *k, int x, int y);
    bool redrawWidget(Karamba *k);
    bool redrawWidgetBackground(Karamba *k);
    bool resizeWidget(Karamba *k, int width, int height);
    bool toggleWidgetRedraw(Karamba *k, bool enable);
};

#endif
